import React, { useState, useRef, useEffect, useCallback } from 'react';
import { Widget } from './components/Widget';
import { Chat } from './components/Chat';
import { ChatConfigurations, MessageList, Observer } from './entities';
import { From, MessageTypes } from './types';
export const ChatComponent = _ref => {
  let {
    configurations,
    ws,
    sessionGateway,
    messagesGateway,
    notificationGateway
  } = _ref;
  const [sessionState, setSessionState] = useState(null);
  const [messageState, setMessageState] = useState(null);
  const isFirstRenderRef = useRef(true);
  const wsRef = useRef(null);
  const onIncomingMessage = useCallback((data, from) => {
    const newMessageList = new MessageList(messageState === null || messageState === void 0 ? void 0 : messageState.messages);
    newMessageList.observers = (messageState === null || messageState === void 0 ? void 0 : messageState.observers) || [];
    newMessageList.addMessage({
      ...data,
      id: data.id || `${Date.now() * Math.random()}`,
      from
    });
    setMessageState(newMessageList);
    if (data.type !== MessageTypes.TYPING) {
      notificationGateway.postLocal(data.message);
    }
  }, [messageState, notificationGateway]);
  sessionGateway.onEndConversation(async () => {
    await sessionGateway.clearSession();
  });
  messagesGateway.onMessage(msg => onIncomingMessage(msg, From.BOT));
  useEffect(() => {
    if (!wsRef.current) {
      wsRef.current = ws;
      wsRef.current.connect();
      wsRef.current.onOpen(async () => {
        await sessionGateway.linkSession();
      });
    }
  }, [ws, sessionGateway]);
  useEffect(() => {
    async function loadData() {
      const [storedSession, storedMessages] = await Promise.all([sessionGateway.getCurrent(), messagesGateway.getMessages()]);
      storedSession.register(new Observer('sendAction', async data => {
        await sessionGateway.sendAction(data);
      }));
      storedSession.register(new Observer('clearSession', async () => {
        await sessionGateway.clearSession();
        const newSession = await sessionGateway.subscribe('');
        newSession.observers = storedSession.observers;
        setSessionState(newSession);
      }));
      setSessionState(storedSession);
      storedMessages.register(new Observer('addMessage', async data => {
        if (data.type !== MessageTypes.TYPING) {
          if (data.from === From.USER) {
            await messagesGateway.sendMessage(data);
          } else {
            await messagesGateway.sendStatus(data.id, 'delivered');
            await messagesGateway.storeMessage(data);
          }
        }
      }));
      setMessageState(storedMessages);
    }
    loadData();
  }, [sessionGateway, messagesGateway]);
  useEffect(() => {
    if (isFirstRenderRef.current) {
      isFirstRenderRef.current = false;
      return;
    }
    const subscribe = async () => {
      const sessionId = (sessionState === null || sessionState === void 0 ? void 0 : sessionState.current) || '';
      await sessionGateway.subscribe(sessionId);
    };
    subscribe();
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [isFirstRenderRef.current]);
  const restartConversation = () => {
    const messageList = new MessageList([]);
    messageList.observers = (messageState === null || messageState === void 0 ? void 0 : messageState.observers) || [];
    setMessageState(messageList);
    messageState === null || messageState === void 0 ? void 0 : messageState.clearMessages();
    sessionState === null || sessionState === void 0 ? void 0 : sessionState.clearSession();
  };
  const [configurationsState, setConfigurationsState] = useState(new ChatConfigurations(configurations.title, configurations.poweredBy, configurations.poweredBy, configurations.settings, configurations.isOpen));
  const toggleIsChatOpen = () => {
    const updatedConfigurations = new ChatConfigurations(configurationsState.title, configurationsState.poweredBy, configurationsState.poweredByUrl, configurationsState.settings, configurationsState.isOpen);
    updatedConfigurations.toggleIsOpen();
    setConfigurationsState(updatedConfigurations);
  };
  const closeChat = () => {
    const updatedConfigurations = new ChatConfigurations(configurationsState.title, configurationsState.poweredBy, configurationsState.poweredByUrl, configurationsState.settings, configurationsState.isOpen);
    updatedConfigurations.close();
    setConfigurationsState(updatedConfigurations);
  };
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Chat, {
    messages: (messageState === null || messageState === void 0 ? void 0 : messageState.messages) || [],
    configurations: configurationsState,
    close: closeChat,
    restartConversation: restartConversation,
    sendMessage: msg => onIncomingMessage(msg, From.USER)
  }), /*#__PURE__*/React.createElement(Widget, {
    onPress: toggleIsChatOpen
  }));
};
//# sourceMappingURL=ChatComponent.js.map